@@ -20,7 +20,9 @@
 
 #include <string.h>
 #include <glib/gi18n-lib.h>
+#ifdef HAVE_LOCALE_H
 #include <locale.h>
+#endif
 #include "gda-parameter-list.h"
 #include "gda-query.h"
 #include "gda-marshal.h"
@@ -37,6 +39,8 @@
 #include "gda-server-provider.h"
 #include "gda-util.h"
 
+extern xmlDtdPtr gda_paramlist_dtd;
+
 /* 
  * Main static functions 
  */
@@ -73,7 +77,7 @@ static gint gda_parameter_list_signals[LAST_SIGNAL] = { 0, 0 };
 /* private structure */
 struct _GdaParameterListPrivate
 {
-	GHashTable *param_default_values;      /* key = param, value = new GdaValue */
+	GHashTable *param_default_values;      /* key = param, value = new GValue */
 	GHashTable *param_default_aliases;     /* key = param, value = alias parameter */
 	GHashTable *aliases_default_param;     /* key = alias parameter, value = param */
 
@@ -218,8 +222,31 @@ gda_parameter_list_new (GSList *params)
 	return paramlist;
 }
 
+static void
+xml_validity_error_func (void *ctx, const char *msg, ...)
+{
+        va_list args;
+        gchar *str, *str2, *newerr;
+
+        va_start (args, msg);
+        str = g_strdup_vprintf (msg, args);
+        va_end (args);
+
+	str2 = *((gchar **) ctx);
+
+        if (str2) {
+                newerr = g_strdup_printf ("%s\n%s", str2, str);
+                g_free (str2);
+        }
+        else
+                newerr = g_strdup (str);
+        g_free (str);
+
+	*((gchar **) ctx) = newerr;
+}
+
 /**
- * gda_parameter_list_new_from_spec
+ * gda_parameter_list_new_from_spec_string
  * @dict: a #GdaDict object, or %NULL
  * @xml_spec: a string
  * @error: a place to store the error, or %NULL
@@ -227,18 +254,106 @@ gda_parameter_list_new (GSList *params)
  * Creates a new #GdaParameterList object from the @xml_spec
  * specifications
  *
- * Returns: a new object, or %NULL if an error occured
+ * Returns: a new object, or %NULL if an error occurred
  */
 GdaParameterList *
-gda_parameter_list_new_from_spec (GdaDict *dict, const gchar *xml_spec, GError **error)
+gda_parameter_list_new_from_spec_string (GdaDict *dict, const gchar *xml_spec, GError **error)
+{
+	xmlDocPtr doc;
+	xmlNodePtr root;
+	GdaParameterList *plist;
+
+	g_return_val_if_fail (!dict || GDA_IS_DICT (dict), NULL);
+
+	/* string parsing */
+	doc = xmlParseMemory (xml_spec, strlen (xml_spec));
+	if (!doc)
+		return NULL;
+
+	{
+                /* doc validation */
+                xmlValidCtxtPtr validc;
+                int xmlcheck;
+		gchar *err_str = NULL;
+		xmlDtdPtr old_dtd = NULL;
+
+                validc = g_new0 (xmlValidCtxt, 1);
+                validc->userData = &err_str;
+                validc->error = xml_validity_error_func;
+                validc->warning = NULL;
+
+                xmlcheck = xmlDoValidityCheckingDefaultValue;
+                xmlDoValidityCheckingDefaultValue = 1;
+
+                /* replace the DTD with ours */
+		old_dtd = doc->intSubset;
+                doc->intSubset = gda_paramlist_dtd;
+
+                if (! xmlValidateDocument (validc, doc)) {
+			doc->intSubset = old_dtd;
+                        xmlFreeDoc (doc);
+                        g_free (validc);
+			
+                        if (err_str) {
+                                g_set_error (error,
+                                             GDA_PARAMETER_LIST_ERROR,
+                                             GDA_PARAMETER_LIST_XML_SPEC_ERROR,
+                                             "XML spec. does not conform to DTD:\n%s", err_str);
+                                g_free (err_str);
+                        }
+                        else
+                                g_set_error (error,
+                                             GDA_PARAMETER_LIST_ERROR,
+                                             GDA_PARAMETER_LIST_XML_SPEC_ERROR,
+                                             "XML spec. does not conform to DTD");
+
+                        xmlDoValidityCheckingDefaultValue = xmlcheck;
+                        return NULL;
+                }
+		doc->intSubset = old_dtd;
+                xmlDoValidityCheckingDefaultValue = xmlcheck;
+                g_free (validc);
+        }
+
+	/* doc is now non NULL */
+	root = xmlDocGetRootElement (doc);
+	if (strcmp (root->name, "data-set-spec") != 0){
+		g_set_error (error, GDA_PARAMETER_LIST_ERROR, GDA_PARAMETER_LIST_XML_SPEC_ERROR,
+			     _("Spec's root node != 'data-set-spec': '%s'"), root->name);
+		return NULL;
+	}
+
+	/* creating parameters */
+	root = root->xmlChildrenNode;
+	while (xmlNodeIsText (root)) 
+		root = root->next; 
+
+	plist = gda_parameter_list_new_from_spec_node (dict, root, error);
+	xmlFreeDoc(doc);
+	return plist;
+}
+
+
+/**
+ * gda_parameter_list_new_from_spec_node
+ * @dict: a #GdaDict object, or %NULL
+ * @xml_spec: a #xmlNodePtr for a &lt;parameters&gt; tag
+ * @error: a place to store the error, or %NULL
+ *
+ * Creates a new #GdaParameterList object from the @xml_spec
+ * specifications
+ *
+ * Returns: a new object, or %NULL if an error occurred
+ */
+GdaParameterList *
+gda_parameter_list_new_from_spec_node (GdaDict *dict, xmlNodePtr xml_spec, GError **error)
 {
 	GdaParameterList *plist = NULL;
 	GSList *params = NULL, *sources = NULL;
 	GSList *list;
 	const gchar *lang;
 
-	xmlDocPtr doc;
-	xmlNodePtr root, cur;
+	xmlNodePtr cur;
 	gboolean allok = TRUE;
 	gchar *str;
 
@@ -258,160 +373,18 @@ gda_parameter_list_new_from_spec (GdaDict *dict, const gchar *xml_spec, GError *
         if (cnc)
                 prov = gda_connection_get_provider_obj (cnc);
 
-	/* string parsing */
-	doc = xmlParseMemory (xml_spec, strlen (xml_spec));
-	if (!doc)
-		return NULL;
-	root = xmlDocGetRootElement (doc);
-	if (strcmp (root->name, "data-set-spec") != 0){
+	if (strcmp (xml_spec->name, "parameters") != 0){
 		g_set_error (error, GDA_PARAMETER_LIST_ERROR, GDA_PARAMETER_LIST_XML_SPEC_ERROR,
-			     _("Spec's root node != 'data-set-spec': '%s'"), root->name);
+			     _("Missing node <parameters>: '%s'"), xml_spec->name);
 		return NULL;
 	}
 
-	/* creating parameters */
-	root = root->xmlChildrenNode;
-	while (xmlNodeIsText (root)) 
-		root = root->next; 
-
-	if (strcmp (root->name, "parameters") != 0){
-		g_set_error (error, GDA_PARAMETER_LIST_ERROR, GDA_PARAMETER_LIST_XML_SPEC_ERROR,
-			     _("Missing node <parameters>: '%s'"), root->name);
-		return NULL;
-	}
-
-	for (cur = root->xmlChildrenNode; cur && allok; cur = cur->next) {
-		if (xmlNodeIsText (cur)) 
-			continue;
-
-		if (!strcmp (cur->name, "parameter")) {
-			GdaParameter *param = NULL;
-			GdaDictType *dtype = NULL;
-			gchar *str, *id;
-			gboolean dtype_created = FALSE;
-			xmlChar *this_lang;
-
-			/* don't care about entries for the wrong locale */
-			this_lang = xmlGetProp (cur, "lang");
-			if (this_lang && strncmp (this_lang, lang, strlen (this_lang))) {
-				g_free (this_lang);
-				continue;
-			}
-
-			/* find if there is already a param with the same ID */
-			id = xmlGetProp (cur, "id");
-			list = params;
-			while (list && !param) {
-				g_object_get (G_OBJECT (list->data), "string_id", &str, NULL);
-				if (str && id && !strcmp (str, id))
-					param = (GdaParameter *) list->data;
-				g_free (str);
-				list = g_slist_next (list);
-			}
-
-			if (param && !this_lang) {
-				g_free (id);
-				g_free (this_lang);
-				continue;
-			}
-			g_free (this_lang);
-			
-
-			/* find data type and create GdaParameter */
-			str = xmlGetProp(cur, "dbmstype");
-			if (str) {
-				dtype = gda_dict_get_data_type_by_name (ASSERT_DICT (dict), str);
-				g_free (str);
-			}
-			if (!dtype) {
-				str = xmlGetProp (cur, "gdatype");
-				if (str) {
-					GdaValueType gtype;
-
-					gtype = gda_type_from_string (str);
-					if (prov) {
-						const gchar *deftype;
-
-						deftype = gda_server_provider_get_default_dbms_type (prov, cnc, 
-												     gtype);
-						if (deftype)
-							dtype = gda_dict_get_data_type_by_name (ASSERT_DICT (dict), 
-												deftype);
-					}	
-
-					if (!dtype) {
-						/* create a GdaDictType for that 'gda-type' */
-						dtype = GDA_DICT_TYPE (gda_dict_type_new (ASSERT_DICT (dict)));
-						gda_dict_type_set_sqlname (dtype, str);
-						gda_dict_type_set_gda_type (dtype, gtype);
-						gda_dict_declare_custom_data_type (ASSERT_DICT (dict), dtype);
-						dtype_created = TRUE;
-					}
-					g_free (str);					
-				}
-			}
-
-			if (!dtype) {
-				str = xmlGetProp(cur, "name");
-				
-				g_set_error (error, GDA_PARAMETER_LIST_ERROR, GDA_PARAMETER_LIST_XML_SPEC_ERROR,
-					     _("Can't find a data type for parameter '%s'"), str ? str : "unnamed");
-				g_free (str);
-
-				allok = FALSE;
-				continue;
-			}
-
-			if (!param) {
-				param = GDA_PARAMETER (g_object_new (GDA_TYPE_PARAMETER, "dict", dict,
-								     "gda_type", gda_dict_type_get_gda_type (dtype),
-								     NULL));
-				params = g_slist_append (params, param);
-			}
-			if (dtype_created)
-				g_object_unref (dtype);
-			
-			/* set parameter's attributes */
-			if (id) {
-				g_object_set (G_OBJECT (param), "string_id", id, NULL);
-				g_free (id);
-			}
-
-			str = xmlGetProp(cur, "name");
-			if (str) {
-				gda_object_set_name (GDA_OBJECT (param), str);
-				g_free (str);
-			}
-			str = xmlGetProp(cur, "descr");
-			if (str) {
-				gda_object_set_description (GDA_OBJECT (param), str);
-				g_free (str);
-			}
-			str = xmlGetProp(cur, "null-ok");
-			if (str) {
-				gda_parameter_set_not_null (param, (*str == 'T') || (*str == 't') ? FALSE : TRUE);
-				g_free (str);
-			}
-			else
-				gda_parameter_set_not_null (param, FALSE);
-			str = xmlGetProp(cur, "plugin");
-			if (str) {
-				g_object_set (G_OBJECT (param), "handler_plugin", str, NULL);
-				g_free (str);
-			}
-
-			str = xmlGetProp(cur, "source");
-			if (str) 
-				g_object_set_data (G_OBJECT (param), "source", str);
-		}
-	}
-
-	/* Parameters' sources, not mandatory */
-	root = root->next;
-	while (xmlNodeIsText (root)) 
-		root = root->next; 
-	if (allok && root && !strcmp (root->name, "sources")){
-		for (cur = root->xmlChildrenNode; (cur != NULL) && allok; cur = cur->next) {
+	/* Parameters' sources, not mandatory: makes the @sources list */
+	cur = xml_spec->next;
+	while (cur && (xmlNodeIsText (cur) || strcmp (cur->name, "sources"))) 
+		cur = cur->next; 
+	if (allok && cur && !strcmp (cur->name, "sources")){
+		for (cur = cur->xmlChildrenNode; (cur != NULL) && allok; cur = cur->next) {
 			if (xmlNodeIsText (cur)) 
 				continue;
 
@@ -438,68 +411,81 @@ gda_parameter_list_new_from_spec (GdaDict *dict, const gchar *xml_spec, GError *
 				}
 			}
 		}
-	}
-	
-	/* affecting parameters' sources fields to each parameter */
-	list = params;
-	while (list && allok) {
-		str = g_object_get_data (G_OBJECT (list->data), "source");
-		if (str) {
-			gchar *ptr1, *ptr2=NULL, *tok;
-			
-			ptr1 = strtok_r (str, ":", &tok);
-			if (ptr1)
-				ptr2 = strtok_r (NULL, ":", &tok);
-			
-			if (ptr1 && ptr2) {
-				GSList *tmp = sources;
-				GdaDataModel *model = NULL;
-				while (tmp && !model) {
-					if (!strcmp (gda_object_get_name (GDA_OBJECT (tmp->data)), ptr1))
-						model = GDA_DATA_MODEL (tmp->data);
-					tmp = g_slist_next (tmp);
-				}
+	}	
 
-				if (model) {
-					gint fno;
+	/* parameters */
+	for (cur = xml_spec->xmlChildrenNode; cur && allok; cur = cur->next) {
+		if (xmlNodeIsText (cur)) 
+			continue;
 
-					fno = atoi (ptr2);
-					if ((fno < 0) ||
-					    (fno >= gda_data_model_get_n_columns (model))) {
-						g_set_error (error, GDA_PARAMETER_LIST_ERROR, GDA_PARAMETER_LIST_XML_SPEC_ERROR,
-							     _("Field number %d not found in source named '%s'"), fno, ptr1); 
-						allok = FALSE;	
-					}
-					else {
-						if (!gda_parameter_restrict_values (GDA_PARAMETER (list->data),
-										    model, fno, NULL))
-							allok = FALSE;
-						else {
-							/* rename the wrapper with the current param's name */
-							g_object_set_data (G_OBJECT (model), "newname", 
-									   (gchar *)gda_object_get_name (GDA_OBJECT (list->data)));
-							g_object_set_data (G_OBJECT (model), "newdescr", 
-									   (gchar *)gda_object_get_description (GDA_OBJECT (list->data)));
-						}
-					}
-				}
-				else {
-					g_set_error (error, GDA_PARAMETER_LIST_ERROR, GDA_PARAMETER_LIST_XML_SPEC_ERROR,
-						     _("Can't find parameter source named '%s'"), ptr1); 
-					allok = FALSE;
-				}
+		if (!strcmp (cur->name, "parameter")) {
+			GdaParameter *param = NULL;
+			GdaDictType *dtype = NULL;
+			gchar *str, *id;
+			gboolean dtype_created = FALSE;
+			xmlChar *this_lang;
+			xmlChar *dbmstype, *gdatype;
+			xmlChar *xmlstr;
+
+			/* don't care about entries for the wrong locale */
+			this_lang = xmlGetProp (cur, "lang");
+			if (this_lang && strncmp (this_lang, lang, strlen (this_lang))) {
+				g_free (this_lang);
+				continue;
 			}
-			else {
+
+			/* find if there is already a param with the same ID */
+			id = xmlGetProp (cur, "id");
+			list = params;
+			while (list && !param) {
+				g_object_get (G_OBJECT (list->data), "string_id", &str, NULL);
+				if (str && id && !strcmp (str, id))
+					param = (GdaParameter *) list->data;
+				g_free (str);
+				list = g_slist_next (list);
+			}
+			if (id) 
+				xmlFree (id);
+
+			if (param && !this_lang) {
+				xmlFree (this_lang);
+				continue;
+			}
+			g_free (this_lang);
+			
+
+			/* find data type and create GdaParameter */
+			dbmstype = xmlGetProp (cur, BAD_CAST "dbmstype");
+			gdatype = xmlGetProp (cur, BAD_CAST "gdatype");
+			dtype = utility_find_or_create_data_type (dict, prov, cnc,
+								  dbmstype, gdatype, &dtype_created);
+			if (dbmstype) xmlFree (dbmstype);
+			if (gdatype) xmlFree (gdatype);
+
+			if (!dtype) {
+				xmlstr = xmlGetProp(cur, BAD_CAST "name");
+				
 				g_set_error (error, GDA_PARAMETER_LIST_ERROR, GDA_PARAMETER_LIST_XML_SPEC_ERROR,
-					     _("Parameter source specification should be \"<source name>:<field pos>\": '%s'"), 
-					     str);
+					     _("Can't find a data type for parameter '%s'"), 
+					     xmlstr ? (gchar *) xmlstr : "unnamed");
+				xmlFree (xmlstr);
+
 				allok = FALSE;
+				continue;
 			}
 
-			g_object_set_data (G_OBJECT (list->data), "source", NULL);
-			g_free (str);
+			if (!param) {
+				param = GDA_PARAMETER (g_object_new (GDA_TYPE_PARAMETER, "dict", dict,
+								     "gda_type", gda_dict_type_get_gda_type (dtype),
+								     NULL));
+				params = g_slist_append (params, param);
+			}
+			if (dtype_created)
+				g_object_unref (dtype);
+			
+			/* set parameter's attributes */
+			utility_parameter_load_attributes (param, cur, sources);
 		}
-		list = g_slist_next (list);
 	}
 
 	/* setting prepared new names from sources (models) */
@@ -521,16 +507,32 @@ gda_parameter_list_new_from_spec (GdaDict *dict, const gchar *xml_spec, GError *
 	/* parameters' values, constraints: TODO */
 	
 	/* GdaParameterList creation */
-	if (allok)
+	if (allok) {
+		xmlChar *prop;;
 		plist = gda_parameter_list_new (params);
 
+		prop = xmlGetProp (xml_spec, "id");
+		if (prop) {
+			gda_object_set_id (GDA_OBJECT (plist), prop);
+			xmlFree (prop);
+		}
+		prop = xmlGetProp (xml_spec, "name");
+		if (prop) {
+			gda_object_set_name (GDA_OBJECT (plist), prop);
+			xmlFree (prop);
+		}
+		prop = xmlGetProp (xml_spec, "descr");
+		if (prop) {
+			gda_object_set_description (GDA_OBJECT (plist), prop);
+			xmlFree (prop);
+		}
+	}
+
 	g_slist_foreach (params, (GFunc) g_object_unref, NULL);
 	g_slist_free (params);
 	g_slist_foreach (sources, (GFunc) g_object_unref, NULL);
 	g_slist_free (sources);
 
-	xmlFreeDoc(doc);
-
 	return plist;
 }
 
@@ -554,7 +556,7 @@ gda_parameter_list_get_length (GdaParameterList *paramlist)
  * gda_parameter_list_get_spec
  * @paramlist: a #GdaParameterList object
  *
- * Get the specification as an XML string. See the gda_parameter_list_new_from_spec()
+ * Get the specification as an XML string. See the gda_parameter_list_new_from_spec_string()
  * form more information about the XML specification string format.
  *
  * Returns: a new string
@@ -568,7 +570,7 @@ gda_parameter_list_get_spec (GdaParameterList *paramlist)
 	int buffersize;
 	GSList *list;
 
-	g_return_val_if_fail (paramlist && GDA_IS_PARAMETER_LIST (paramlist), NULL);
+	g_return_val_if_fail (GDA_IS_PARAMETER_LIST (paramlist), NULL);
 
 	doc = xmlNewDoc ("1.0");
 	g_return_val_if_fail (doc, NULL);
@@ -605,8 +607,8 @@ gda_parameter_list_get_spec (GdaParameterList *paramlist)
 /* 		} */
 		xmlSetProp (node, "gdatype", gda_type_to_string (gda_parameter_get_gda_type (param)));
 
-		xmlSetProp (node, "null-ok", gda_parameter_get_not_null (param) ? "FALSE" : "TRUE");
-		g_object_get (G_OBJECT (param), "handler_plugin", &str, NULL);
+		xmlSetProp (node, "nullok", gda_parameter_get_not_null (param) ? "FALSE" : "TRUE");
+		g_object_get (G_OBJECT (param), "entry_plugin", &str, NULL);
 		if (str) {
 			xmlSetProp (node, "plugin", str);
 			g_free (str);
@@ -955,8 +957,8 @@ compute_ref_columns_index (GdaParameterListSource *source)
 void
 gda_parameter_list_add_param (GdaParameterList *paramlist, GdaParameter *param)
 {
-	g_return_if_fail (paramlist && GDA_IS_PARAMETER_LIST (paramlist));
-	g_return_if_fail (param && GDA_IS_PARAMETER (param));
+	g_return_if_fail (GDA_IS_PARAMETER_LIST (paramlist));
+	g_return_if_fail (GDA_IS_PARAMETER (param));
 
 	gda_parameter_list_real_add_param (paramlist, param);
 	compute_public_data (paramlist);
@@ -1019,17 +1021,93 @@ gda_parameter_list_real_add_param (GdaParameterList *paramlist, GdaParameter *pa
 }
 
 /**
- * gda_parameter_list_merge_paramlist_params
+ * gda_parameter_list_add_param_from_string
+ * @paramlist: a #GdaParameterList object
+ * @name: the name to give to the new parameter
+ * @type: the type of parameter to add
+ * @str: the string representation of the parameter
+ *
+ * Creates and adds a new #GdaParameter to @paramlist. The ID and name of the new parameter
+ * are set as @name. The parameter's value is set from @str.
+ *
+ * Returns: the new #GdaParameter for information, or %NULL if an error occurred
+ */
+GdaParameter *
+gda_parameter_list_add_param_from_string (GdaParameterList *paramlist, const gchar *name,
+					  GType type, const gchar *str)
+{
+	GdaParameter *param;
+	GdaDict *dict;
+
+	g_return_val_if_fail (GDA_IS_PARAMETER_LIST (paramlist), NULL);
+	g_return_val_if_fail (paramlist->priv, NULL);
+
+	dict = gda_object_get_dict ((GdaObject*) paramlist);
+	param = g_object_new (GDA_TYPE_PARAMETER, "dict", dict, "gda_type", type, NULL);
+	g_return_val_if_fail (param, NULL);
+
+	if (! gda_parameter_set_value_str (param, str)) {
+		g_object_unref (param);
+		g_warning (_("Could not add parameter of type %s with value '%s'"), 
+			   gda_type_to_string (type), str);
+		return NULL;
+	}
+
+	gda_object_set_name ((GdaObject *) param, name);
+	gda_object_set_id ((GdaObject *) param, name);
+	gda_parameter_list_add_param (paramlist, param);
+	g_object_unref (param);
+
+	return param;
+}
+
+/**
+ * gda_parameter_list_add_param_from_value
+ * @paramlist: a #GdaParameterList object
+ * @name: the name to give to the new parameter
+ * @value: the value to give to the new parameter, must not be NULL or of type null
+ *
+ * Creates and adds a new #GdaParameter to @paramlist. The ID and name of the new parameter
+ * are set as @name. The parameter's value is a copy of @value.
+ *
+ * Returns: the new #GdaParameter for information, or %NULL if an error occurred
+ */
+GdaParameter *
+gda_parameter_list_add_param_from_value (GdaParameterList *paramlist, const gchar *name, GValue *value)
+{
+	GdaParameter *param;
+	GdaDict *dict;
+
+	g_return_val_if_fail (GDA_IS_PARAMETER_LIST (paramlist), NULL);
+	g_return_val_if_fail (paramlist->priv, NULL);
+	g_return_val_if_fail (G_IS_VALUE (value), NULL);
+
+	dict = gda_object_get_dict ((GdaObject*) paramlist);
+	param = g_object_new (GDA_TYPE_PARAMETER, "dict", dict, "gda_type", G_VALUE_TYPE (value), NULL);
+	g_return_val_if_fail (param, NULL);
+
+	gda_parameter_set_value (param, value);
+	gda_object_set_name ((GdaObject *) param, name);
+	gda_object_set_id ((GdaObject *) param, name);
+	gda_parameter_list_add_param (paramlist, param);
+	g_object_unref (param);
+
+	return param;
+}
+
+
+/**
+ * gda_parameter_list_merge
  * @paramlist: a #GdaParameterList object
  * @paramlist_to_merge: a #GdaParameterList object
  *
  * Add to @paramlist all the parameters of @paramlist_to_merge.
  */
 void
-gda_parameter_list_merge_paramlist_params (GdaParameterList *paramlist, GdaParameterList *paramlist_to_merge)
+gda_parameter_list_merge (GdaParameterList *paramlist, GdaParameterList *paramlist_to_merge)
 {
 	GSList *params = paramlist_to_merge->parameters;
-	g_return_if_fail (paramlist && GDA_IS_PARAMETER_LIST (paramlist));
+	g_return_if_fail (GDA_IS_PARAMETER_LIST (paramlist));
 	g_return_if_fail (paramlist_to_merge && GDA_IS_PARAMETER_LIST (paramlist_to_merge));
 
 	while (params) {
@@ -1128,13 +1206,13 @@ gda_parameter_list_is_coherent (GdaParameterList *paramlist, GError **error)
 			if (!g_slist_find (paramlist->nodes_list, list2->data)) {
 				g_set_error (error, GDA_PARAMETER_LIST_ERROR, 0,
 					     _("GdaParameterListSource references a GdaParameterListNode"
-					       "not found in the nodes list"));
+					       " not found in the nodes list"));
 				return FALSE;
 			}
 			if (GDA_PARAMETER_LIST_NODE (list2->data)->source_model != source->data_model) {
 				g_set_error (error, GDA_PARAMETER_LIST_ERROR, 0,
 					     _("GdaParameterListSource references a GdaParameterListNode"
-					       "which does not use the same data model"));
+					       " which does not use the same data model"));
 				return FALSE;
 			}
 			list2 = g_slist_next (list2);
@@ -1185,7 +1263,7 @@ gda_parameter_list_is_valid (GdaParameterList *paramlist)
 	GSList *params;
 	gboolean retval = TRUE;
 
-	g_return_val_if_fail (paramlist && GDA_IS_PARAMETER_LIST (paramlist), FALSE);
+	g_return_val_if_fail (GDA_IS_PARAMETER_LIST (paramlist), FALSE);
 	g_return_val_if_fail (paramlist->priv, FALSE);
 
 	params = paramlist->parameters;
@@ -1221,7 +1299,7 @@ gda_parameter_list_find_param (GdaParameterList *paramlist, const gchar *param_n
 	gchar *string_id;
 	gchar *pname;
 
-	g_return_val_if_fail (paramlist && GDA_IS_PARAMETER_LIST (paramlist), NULL);
+	g_return_val_if_fail (GDA_IS_PARAMETER_LIST (paramlist), NULL);
 	g_return_val_if_fail (paramlist->priv, NULL);
 
 	list = paramlist->parameters;
@@ -1256,7 +1334,7 @@ gda_parameter_list_find_param_for_user (GdaParameterList *paramlist, GdaObject *
 	GdaParameter *param = NULL;
 	GSList *list;
 
-	g_return_val_if_fail (paramlist && GDA_IS_PARAMETER_LIST (paramlist), NULL);
+	g_return_val_if_fail (GDA_IS_PARAMETER_LIST (paramlist), NULL);
 	g_return_val_if_fail (paramlist->priv, NULL);
 
 	list = paramlist->parameters;
@@ -1397,7 +1475,7 @@ gda_parameter_list_find_source (GdaParameterList *paramlist, GdaDataModel *model
  * gda_parameter_list_set_param_default_value
  * @paramlist: a #GdaParameterList object
  * @param: a #GdaParameter object, managed by @paramlist
- * @value: a #GdaValue, of the same type as @param, or %NULL
+ * @value: a #GValue, of the same type as @param, or %NULL
  *
  * Stores @value in @paramlist to make it possible for @paramlist's users to find a default value
  * for @param when one is required, instead of %NULL.
@@ -1406,17 +1484,17 @@ gda_parameter_list_find_source (GdaParameterList *paramlist, GdaDataModel *model
  * gda_parameter_list_get_param_default_value() is used is up to @paramlist's user.
  */
 void
-gda_parameter_list_set_param_default_value (GdaParameterList *paramlist, GdaParameter *param, const GdaValue *value)
+gda_parameter_list_set_param_default_value (GdaParameterList *paramlist, GdaParameter *param, const GValue *value)
 {
-	g_return_if_fail (paramlist && GDA_IS_PARAMETER_LIST (paramlist));
-	g_return_if_fail (param && GDA_IS_PARAMETER (param));
+	g_return_if_fail (GDA_IS_PARAMETER_LIST (paramlist));
+	g_return_if_fail (GDA_IS_PARAMETER (param));
 	g_return_if_fail (g_slist_find (paramlist->parameters, param));
 
-	if (value && ! gda_value_is_null ((GdaValue *) value)) {
-		g_return_if_fail (gda_value_get_type ((GdaValue *) value) ==
+	if (value && ! gda_value_is_null ((GValue *) value)) {
+		g_return_if_fail (G_VALUE_TYPE ((GValue *) value) ==
 				  gda_parameter_get_gda_type (param));
 		g_hash_table_insert (paramlist->priv->param_default_values, param, 
-				     gda_value_copy ((GdaValue *) value));
+				     gda_value_copy ((GValue *) value));
 	}
 	else
 		g_hash_table_remove (paramlist->priv->param_default_values, param);
@@ -1441,8 +1519,8 @@ gda_parameter_list_set_param_default_alias (GdaParameterList *paramlist, GdaPara
 {
 	GdaParameter *oldalias;
 
-	g_return_if_fail (paramlist && GDA_IS_PARAMETER_LIST (paramlist));
-	g_return_if_fail (param && GDA_IS_PARAMETER (param));
+	g_return_if_fail (GDA_IS_PARAMETER_LIST (paramlist));
+	g_return_if_fail (GDA_IS_PARAMETER (param));
 	g_return_if_fail (g_slist_find (paramlist->parameters, param));
 
 	/* remove the old alias if there was one */
@@ -1496,15 +1574,15 @@ foreach_finalize_alias (GdaParameter *param, GdaParameter *alias, GdaParameterLi
  * using gda_parameter_list_set_param_default_value() or to store a #GdaParameter reference from which to get
  * a value using gda_parameter_list_set_param_default_alias().
  *
- * Returns: a #GdaValue, or %NULL.
+ * Returns: a #GValue, or %NULL.
  */
-const GdaValue *
+const GValue *
 gda_parameter_list_get_param_default_value  (GdaParameterList *paramlist, GdaParameter *param)
 {
-	const GdaValue *value;
-	g_return_val_if_fail (paramlist && GDA_IS_PARAMETER_LIST (paramlist), NULL);
+	const GValue *value;
+	g_return_val_if_fail (GDA_IS_PARAMETER_LIST (paramlist), NULL);
 	g_return_val_if_fail (paramlist->priv, NULL);
-	g_return_val_if_fail (param && GDA_IS_PARAMETER (param), NULL);
+	g_return_val_if_fail (GDA_IS_PARAMETER (param), NULL);
 
 	value = g_hash_table_lookup (paramlist->priv->param_default_values, param);
 	if (value)
@@ -1527,7 +1605,7 @@ gda_parameter_list_dump (GdaParameterList *paramlist, guint offset)
 	gchar *str;
         guint i;
 	
-	g_return_if_fail (paramlist && GDA_IS_PARAMETER_LIST (paramlist));
+	g_return_if_fail (GDA_IS_PARAMETER_LIST (paramlist));
 
         /* string for the offset */
         str = g_new0 (gchar, offset+1);
@@ -1573,7 +1651,6 @@ gda_parameter_list_dump (GdaParameterList *paramlist, guint offset)
 /* 			} */
 /* 			list = g_slist_next (list); */
 /* 		} */
-		TO_IMPLEMENT;
 	}
         else
                 g_print ("%s" D_COL_ERR "Using finalized object %p" D_COL_NOR, str, paramlist);
