@@ -281,6 +281,10 @@ gda_query_target_new_copy (GdaQueryTarget *orig)
 		GdaObjectRefType ref_type;
 		GType ref_gtype;
 
+		ref_str = gda_object_ref_get_ref_object_name (orig->priv->entity_ref);
+		if (ref_str)
+			g_object_set (G_OBJECT (target->priv->entity_ref), "obj_name", ref_str, NULL);
+
 		ref_str = gda_object_ref_get_ref_name (orig->priv->entity_ref, &ref_gtype, &ref_type);
 		if (ref_str)
 			gda_object_ref_set_ref_name (target->priv->entity_ref, ref_gtype, ref_type, ref_str);
@@ -500,8 +504,12 @@ gda_query_target_get_represented_table_name (GdaQueryTarget *target)
 		else
 			return NULL;
 	}
-	else
-		return gda_object_ref_get_ref_name (target->priv->entity_ref, NULL, NULL);
+	else {
+		if (gda_object_ref_get_ref_object_name (target->priv->entity_ref))
+			return gda_object_ref_get_ref_object_name (target->priv->entity_ref);
+		else
+			return gda_object_ref_get_ref_name (target->priv->entity_ref, NULL, NULL);
+	}
 }
 
 /**
@@ -878,8 +886,21 @@ gda_query_target_render_as_sql (GdaRenderer *iface, GdaParameterList *context, g
 
 	entity = gda_query_target_get_represented_entity (target);
 	if (entity) {
-		if (GDA_IS_DICT_TABLE (entity))
-			string = g_string_new (gda_object_get_name (GDA_OBJECT (entity)));
+		if (GDA_IS_DICT_TABLE (entity)) {
+			gchar *tmp, *tname;
+
+			tname = (gchar *) gda_object_get_name (GDA_OBJECT (entity));
+			tmp = g_utf8_strdown (tname, -1);
+			if (((*tmp <= '9') && (*tmp >= '0')) || 
+			    strcmp (tmp, tname)) {
+				g_free (tmp);
+				tmp = tname;
+				tname = g_strdup_printf ("\"%s\"", tmp);
+			}
+			else
+				g_free (tmp);
+			string = g_string_new (tname);
+		}
 		
 		if (GDA_IS_QUERY (entity)) {
 			string = g_string_new ("(");
@@ -916,12 +937,14 @@ gda_query_target_render_as_sql (GdaRenderer *iface, GdaParameterList *context, g
 		cnc = gda_dict_get_connection (dict);
 		if (cnc)
 			sinfo = gda_connection_get_infos (cnc);
-		
-		if (!sinfo || (sinfo->alias_needs_as_keyword))
-			g_string_append (string, " AS ");
-		else
-			g_string_append_c (string, ' ');
-		g_string_append (string, gda_query_target_get_alias (target));
+	
+		if (!sinfo || sinfo->supports_alias) {
+			if (!sinfo || sinfo->alias_needs_as_keyword)
+				g_string_append (string, " AS ");
+			else
+				g_string_append_c (string, ' ');
+			g_string_append (string, gda_query_target_get_alias (target));
+		}
 		
 		str = string->str;
 	}
