@@ -1,5 +1,5 @@
 /* GDA library
- * Copyright (C) 1998 - 2005 The GNOME Foundation.
+ * Copyright (C) 1998 - 2006 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -27,6 +27,7 @@
 #include <libgda/gda-server-provider-extra.h>
 #include <libgda/gda-server-provider-private.h>
 #include <libgda/gda-data-handler.h>
+#include <libgda/gda-parameter-list.h>
 #include <string.h>
 #include <glib/gi18n-lib.h>
 
@@ -70,28 +71,22 @@ gda_server_provider_class_init (GdaServerProviderClass *klass)
 	klass->get_version = NULL;
 	klass->get_server_version = NULL;
 	klass->get_info = NULL;
-	klass->supports = NULL;
+	klass->supports_feature = NULL;
 	klass->get_schema = NULL;
 
 	klass->get_data_handler = NULL;
 	klass->string_to_value = NULL;
 
 	klass->open_connection = NULL;
-	klass->reset_connection = NULL;
 	klass->close_connection = NULL;
 	klass->get_database = NULL;
 	klass->change_database = NULL;
 
-	klass->get_specs = NULL;
-	klass->perform_action_params = NULL;
+	klass->supports_operation = NULL;
+	klass->create_operation = NULL;
+	klass->render_operation = NULL;
+	klass->perform_operation = NULL;
 
-	klass->create_database_cnc = NULL;
-	klass->drop_database_cnc = NULL;
-
-	klass->create_table = NULL;
-	klass->drop_table = NULL;
-	klass->create_index = NULL;
-	klass->drop_index = NULL;
 	klass->execute_command = NULL;
 	klass->get_last_insert_id = NULL;
 	klass->begin_transaction = NULL;
@@ -184,7 +179,7 @@ gda_server_provider_get_version (GdaServerProvider *provider)
  * Retreive some information specific to the provider. The returned #GdaServerProviderInfo
  * structure must not be modified
  *
- * Returns: a #GdaServerProviderInfo pointer or %NULL if an error occured
+ * Returns: a #GdaServerProviderInfo pointer or %NULL if an error occurred
  */
 GdaServerProviderInfo *
 gda_server_provider_get_info (GdaServerProvider *provider, GdaConnection *cnc)
@@ -265,25 +260,6 @@ gda_server_provider_open_connection (GdaServerProvider *provider,
 	return retcode;
 }
 
-/**
- * gda_server_provider_reset_connection:
- * @provider: A #GdaServerProvider object.
- * @cnc: The connection to be reset.
- *
- * Calls the reset_connection method implementation on the given #GdaServerProvider.
- *
- * Returns: %TRUE if successful, %FALSE otherwise.
- */
-gboolean
-gda_server_provider_reset_connection (GdaServerProvider *provider, GdaConnection *cnc)
-{
-	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (CLASS (provider)->reset_connection != NULL, FALSE);
-
-	return CLASS (provider)->reset_connection (provider, cnc);
-}
-
 /**
  * gda_server_provider_close_connection
  * @provider:
@@ -374,220 +350,269 @@ gda_server_provider_change_database (GdaServerProvider *provider,
 }
 
 /**
- * gda_server_provider_get_specs
+ * gda_server_provider_supports_operation
  * @provider: a #GdaServerProvider object
- * @action_type: what action the specs are for
+ * @cnc: a #GdaConnection object which would be used to perform an action
+ * @type: the type of operation requested
+ * @options: a list of named parameters, or %NULL
  *
- * Fetch a list of parameters required to create a database for the specific
- * @provider provider.
+ * Tells if @provider supports the @type of operation on the @cnc connection, using the
+ * (optional) @options parameters.
  *
- * The list of parameters is returned as an XML string listing each paraleter, its type,
- * its name, etc.
- *
- * Returns: a new XML string, or %NULL if @provider does not implement that method.
+ * Returns: TRUE if the operation is supported
  */
-gchar *
-gda_server_provider_get_specs  (GdaServerProvider *provider,
-				GdaClientSpecsType action_type)
+gboolean
+gda_server_provider_supports_operation (GdaServerProvider *provider, GdaConnection *cnc, 
+					GdaServerOperationType type, GdaParameterList *options)
 {
-	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), NULL);
+	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
+	if (cnc) {
+		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+		g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	}
+	if (CLASS (provider)->supports_operation)
+		return CLASS (provider)->supports_operation (provider, cnc, type, options);
+	else
+		return FALSE;
+}
 
-	if (CLASS (provider)->get_specs)
-		return CLASS (provider)->get_specs (provider, action_type);
+typedef struct {
+	gchar                      *path;
+	GdaServerOperationNodeType  node_type;
+	GType                       data_type;
+} OpReq;
+
+static OpReq op_req_CREATE_DB [] = {
+	{"/DB_DEF_P",               GDA_SERVER_OPERATION_NODE_PARAMLIST, 0},
+	{"/DB_DEF_P/DB_NAME",       GDA_SERVER_OPERATION_NODE_PARAM, G_TYPE_STRING},
+	{NULL}
+};
+
+static OpReq op_req_DROP_DB [] = {
+	{"/DB_DESC_P",               GDA_SERVER_OPERATION_NODE_PARAMLIST, 0},
+	{"/DB_DESC_P/DB_NAME",       GDA_SERVER_OPERATION_NODE_PARAM, G_TYPE_STRING},
+	{NULL}
+};
+
+static OpReq op_req_CREATE_TABLE [] = {
+	{"/TABLE_DEF_P",               GDA_SERVER_OPERATION_NODE_PARAMLIST, 0},
+	{"/TABLE_DEF_P/TABLE_NAME",    GDA_SERVER_OPERATION_NODE_PARAM, G_TYPE_STRING},
+	{"/FIELDS_A",                  GDA_SERVER_OPERATION_NODE_DATA_MODEL, 0},
+	{"/FIELDS_A/@COLUMN_NAME",     GDA_SERVER_OPERATION_NODE_DATA_MODEL_COLUMN, G_TYPE_STRING},
+	{"/FIELDS_A/@COLUMN_TYPE",     GDA_SERVER_OPERATION_NODE_DATA_MODEL_COLUMN, G_TYPE_STRING},
+	{NULL}
+};
+
+static OpReq op_req_DROP_TABLE [] = {
+	{"/TABLE_DESC_P/TABLE_NAME",   GDA_SERVER_OPERATION_NODE_PARAM, G_TYPE_STRING},
+	{NULL}
+};
+
+static OpReq op_req_RENAME_TABLE [] = {
+	{"/TABLE_DESC_P",                  GDA_SERVER_OPERATION_NODE_PARAMLIST, 0},
+	{"/TABLE_DESC_P/TABLE_NAME",       GDA_SERVER_OPERATION_NODE_PARAM, G_TYPE_STRING},
+	{"/TABLE_DESC_P/TABLE_NEW_NAME",   GDA_SERVER_OPERATION_NODE_PARAM, G_TYPE_STRING},
+	{NULL}
+};
+
+static OpReq op_req_ADD_COLUMN [] = {
+	{"/COLUMN_DEF_P",               GDA_SERVER_OPERATION_NODE_PARAMLIST, 0},
+	{"/COLUMN_DEF_P/TABLE_NAME",    GDA_SERVER_OPERATION_NODE_PARAM, G_TYPE_STRING},
+	{"/COLUMN_DEF_P/COLUMN_NAME",   GDA_SERVER_OPERATION_NODE_PARAM, G_TYPE_STRING},
+	{"/COLUMN_DEF_P/COLUMN_TYPE",   GDA_SERVER_OPERATION_NODE_PARAM, G_TYPE_STRING},
+	{NULL}
+};
+
+static OpReq op_req_DROP_COLUMN [] = {
+	{"/COLUMN_DESC_P",               GDA_SERVER_OPERATION_NODE_PARAMLIST, 0},
+	{"/COLUMN_DESC_P/TABLE_NAME",    GDA_SERVER_OPERATION_NODE_PARAM, G_TYPE_STRING},
+	{"/COLUMN_DESC_P/COLUMN_NAME",   GDA_SERVER_OPERATION_NODE_PARAM, G_TYPE_STRING},
+	{NULL}
+};
+
+static OpReq op_req_CREATE_INDEX [] = {
+	{"/INDEX_DEF_P/INDEX_NAME",       GDA_SERVER_OPERATION_NODE_PARAM, G_TYPE_STRING},
+	{"/INDEX_DEF_P/INDEX_ON_TABLE",   GDA_SERVER_OPERATION_NODE_PARAM, G_TYPE_STRING},
+	{"/INDEX_FIELDS_S",               GDA_SERVER_OPERATION_NODE_SEQUENCE, 0},
+	{NULL}
+};
+
+static OpReq op_req_DROP_INDEX [] = {
+	{"/INDEX_DESC_P/INDEX_NAME",   GDA_SERVER_OPERATION_NODE_PARAM, G_TYPE_STRING},
+	{NULL}
+};
+
+/**
+ * gda_server_provider_create_operation
+ * @provider: a #GdaServerProvider object
+ * @cnc: a #GdaConnection object which will be used to perform an action
+ * @type: the type of operation requested
+ * @options: an optional list of parameters
+ * @error: a place to store an error, or %NULL
+ *
+ * Creates a new #GdaServerOperation object which can be modified in order to perform the @type type of
+ * action. The @options can contain:
+ * <itemizedlist>
+ *  <listitem>parameters which ID is a path in the resulting GdaServerOperation object, to initialize some value</listitem>
+ *  <listitem>parameters which may change the contents of the GdaServerOperation, see <link linkend="gda-server-op-information">this section</link> for more information</listitem>
+ * </itemizedlist>
+ *
+ * Returns: a new #GdaServerOperation object, or %NULL in the provider does not support the @type type
+ * of operation or if an error occurred
+ */
+GdaServerOperation *
+gda_server_provider_create_operation (GdaServerProvider *provider, GdaConnection *cnc, 
+				      GdaServerOperationType type, 
+				      GdaParameterList *options, GError **error)
+{
+	OpReq **op_req_table = NULL;
+
+	if (! op_req_table) {
+		op_req_table = g_new0 (OpReq *, GDA_SERVER_OPERATION_NB);
+
+		op_req_table [GDA_SERVER_OPERATION_CREATE_DB] = op_req_CREATE_DB;
+		op_req_table [GDA_SERVER_OPERATION_DROP_DB] = op_req_DROP_DB;
+		
+		op_req_table [GDA_SERVER_OPERATION_CREATE_TABLE] = op_req_CREATE_TABLE;
+		op_req_table [GDA_SERVER_OPERATION_DROP_TABLE] = op_req_DROP_TABLE;
+		op_req_table [GDA_SERVER_OPERATION_RENAME_TABLE] = op_req_RENAME_TABLE;
+
+		op_req_table [GDA_SERVER_OPERATION_ADD_COLUMN] = op_req_ADD_COLUMN;
+		op_req_table [GDA_SERVER_OPERATION_DROP_COLUMN] = op_req_DROP_COLUMN;
+
+		op_req_table [GDA_SERVER_OPERATION_CREATE_INDEX] = op_req_CREATE_INDEX;
+		op_req_table [GDA_SERVER_OPERATION_DROP_INDEX] = op_req_DROP_INDEX;
+	}
+
+	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), NULL);
+	if (cnc) {
+		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+		g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, NULL);
+	}
+	if (CLASS (provider)->create_operation) {
+		GdaServerOperation *op;
+		GdaServerProviderInfo *pinfo;
+
+		pinfo = gda_server_provider_get_info (provider, cnc);
+		op = CLASS (provider)->create_operation (provider, cnc, type, options, error);
+		if (op) {
+			/* test op's conformance */
+			OpReq *opreq = op_req_table [type];
+			while (opreq && opreq->path) {
+				GdaServerOperationNodeType node_type;
+				node_type = gda_server_operation_get_node_type (op, opreq->path, NULL);
+				if (node_type == GDA_SERVER_OPERATION_NODE_UNKNOWN) 
+					g_warning (_("Provider %s created a GdaServerOperation without node for '%s'"),
+						   pinfo->provider_name, opreq->path);
+				else 
+					if (node_type != opreq->node_type)
+						g_warning (_("Provider %s created a GdaServerOperation with wrong node type for '%s'"),
+							   pinfo->provider_name, opreq->path);
+				opreq += 1;
+			}
+
+			if (options) {
+				/* pre-init parameters depending on the @options argument */
+				GSList *list;
+				xmlNodePtr top, node;
+
+				top =  xmlNewNode (NULL, BAD_CAST "serv_op_data");
+				for (list = options->parameters; list; list = list->next) {
+					const gchar *id;
+					gchar *str = NULL;
+					const GValue *value;
+
+					id = gda_object_get_id (GDA_OBJECT (list->data));
+					value = gda_parameter_get_value (GDA_PARAMETER (list->data));
+					if (value)
+						str = gda_value_stringify (value);
+					node = xmlNewChild (top, NULL, BAD_CAST "op_data", BAD_CAST str);
+					g_free (str);
+					xmlSetProp (node, BAD_CAST "path", BAD_CAST id);
+				}
+
+				if (! gda_server_operation_load_data_from_xml (op, top, error))
+					g_warning ("Incorrect options");
+				xmlFreeNode (top);
+			}
+		}
+		return op;
+	}
 	else
 		return NULL;
 }
 
 /**
- * gda_server_provider_perform_action_params
+ * gda_server_provider_render_operation
  * @provider: a #GdaServerProvider object
- * @params: a list of parameters required to create a database
- * @action_type: action to perform
+ * @cnc: a #GdaConnection object which will be used to perform an action
+ * @op: a #GdaServerOperation object
  * @error: a place to store an error, or %NULL
  *
- * Performs a specific action specified by the @action_type argument
- * using the parameters listed in @params (the list of parameters may have
- * been obtained using the gda_server_provider_get_specs() method).
+ * Creates an SQL statement (possibly using some specific extensions of the DBMS) corresponding to the
+ * @op operation.
  *
- * Returns: TRUE if no error occured
+ * Returns: a new string, or %NULL if an error occurred or operation cannot be rendered as SQL.
  */
-gboolean
-gda_server_provider_perform_action_params (GdaServerProvider *provider, 
-					   GdaParameterList *params, 
-					   GdaClientSpecsType action_type,
-					   GError **error)
+gchar *
+gda_server_provider_render_operation (GdaServerProvider *provider, GdaConnection *cnc, 
+				      GdaServerOperation *op, GError **error)
 {
-	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
-
-	if (CLASS (provider)->perform_action_params)
-		return CLASS (provider)->perform_action_params (provider, params, action_type,
-								error);
-	else {
-		gchar *str;
-
-		str = g_strdup_printf (_("Provider does not support the '%s()' method"), 
-				       "perform_action_params");
-		g_set_error (error, 0, 0, str);
-		g_free (str);
-		return FALSE;
+	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), NULL);
+	if (cnc) {
+		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+		g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, NULL);
 	}
-}
-
-
-/**
- * gda_server_provider_create_database_cnc
- * @provider: a #GdaServerProvider object.
- * @name: database name.
- * @cnc: a #GdaConnection object.
- *
- * Creates a database named @name using the @cnc connection.
- *
- * Returns: %TRUE if successful, %FALSE otherwise.
- */
-gboolean
-gda_server_provider_create_database_cnc (GdaServerProvider *provider,
-					 GdaConnection *cnc,
-					 const gchar *name)
-{
-	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (name != NULL, FALSE);
-	g_return_val_if_fail (CLASS (provider)->create_database_cnc != NULL, FALSE);
-
-	return CLASS (provider)->create_database_cnc (provider, cnc, name);
+	if (CLASS (provider)->render_operation)
+		return CLASS (provider)->render_operation (provider, cnc, op, error);
+	else
+		return NULL;
 }
 
 /**
- * gda_server_provider_drop_database_cnc
- * @provider: a #GdaServerProvider object.
- * @cnc: a #GdaConnection object.
- * @name: database name.
+ * gda_server_provider_perform_operation
+ * @provider: a #GdaServerProvider object
+ * @cnc: a #GdaConnection object which will be used to perform an action
+ * @op: a #GdaServerOperation object
+ * @error: a place to store an error, or %NULL
  *
- * Destroy the database named @name using the @cnc connection.
+ * Performs the operation described by @op.
  *
- * Returns: %TRUE if successful, %FALSE otherwise.
+ * Returns: TRUE if no error occurred
  */
 gboolean
-gda_server_provider_drop_database_cnc (GdaServerProvider *provider,
-				       GdaConnection *cnc,
-				       const gchar *name)
+gda_server_provider_perform_operation (GdaServerProvider *provider, GdaConnection *cnc, 
+				       GdaServerOperation *op, GError **error)
 {
 	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (name != NULL, FALSE);
-	g_return_val_if_fail (CLASS (provider)->drop_database_cnc != NULL, FALSE);
+	if (cnc) {
+		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+		g_return_val_if_fail (gda_connection_get_provider_obj (cnc) == provider, FALSE);
+	}
+	if (CLASS (provider)->perform_operation)
+		return CLASS (provider)->perform_operation (provider, cnc, op, error);
+	else {
+		/* use the SQL from the provider to perform the action */
+		gchar *sql;
+		GdaCommand *cmd;
+		GdaDataModel *model;
 
-	return CLASS (provider)->drop_database_cnc (provider, cnc, name);
-}
+		sql = gda_server_provider_render_operation (provider, cnc, op, error);
+		if (!sql)
+			return FALSE;
 
-
-/**
- * gda_server_provider_create_table:
- * @provider: a #GdaServerProvider object.
- * @cnc: a #GdaConnection object.
- * @table_name: name of the table to create.
- * @attributes_list: list of #GdaColumn for all fields in the table.
- * @index_list: list of #GdaDataModelIndex for all (additional) indexes in the table.
- *
- * Proxy the call to the create_table method on the #GdaServerProvider class
- * to the corresponding provider.
- *
- * Returns: %TRUE if successful, %FALSE otherwise.
- */
-gboolean
-gda_server_provider_create_table (GdaServerProvider *provider,
-				  GdaConnection *cnc,
-				  const gchar *table_name,
-				  const GList *attributes_list,
-				  const GList *index_list)
-{
-	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (table_name != NULL, FALSE);
-	g_return_val_if_fail (attributes_list != NULL, FALSE);
-	g_return_val_if_fail (CLASS (provider)->create_table != NULL, FALSE);
-
-	return CLASS (provider)->create_table (provider, cnc, table_name, attributes_list, index_list);
-}
-
-/**
- * gda_server_provider_drop_table:
- * @provider: a #GdaServerProvider object.
- * @cnc: a #GdaConnection object.
- * @table_name: name of the table to remove.
- *
- * Proxy the call to the drop_table method on the #GdaServerProvider class
- * to the corresponding provider.
- *
- * Returns: %TRUE if successful, %FALSE otherwise.
- */
-gboolean
-gda_server_provider_drop_table (GdaServerProvider *provider,
-				GdaConnection *cnc,
-				const gchar *table_name)
-{
-	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (table_name != NULL, FALSE);
-	g_return_val_if_fail (CLASS (provider)->drop_table != NULL, FALSE);
-
-	return CLASS (provider)->drop_table (provider, cnc, table_name);
-}
-
-/**
- * gda_server_provider_create_index:
- * @provider: a #GdaServerProvider object.
- * @cnc: a #GdaConnection object.
- * @index: a #GdaDataModelIndex object containing all index information.
- * @table_name: name of the table to create index for.
- *
- * Proxy the call to the create_index method on the #GdaServerProvider class
- * to the corresponding provider.
- *
- * Returns: %TRUE if successful, %FALSE otherwise.
- */
-gboolean
-gda_server_provider_create_index (GdaServerProvider *provider,
-				  GdaConnection *cnc,
-				  const GdaDataModelIndex *index,
-				  const gchar *table_name)
-{
-	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (table_name != NULL, FALSE);
-	g_return_val_if_fail (index != NULL, FALSE);
-	g_return_val_if_fail (CLASS (provider)->create_index != NULL, FALSE);
-
-	return CLASS (provider)->create_index (provider, cnc, index, table_name);
-}
-
-/**
- * gda_server_provider_drop_index:
- * @provider: a #GdaServerProvider object.
- * @cnc: a #GdaConnection object.
- * @index_name: name of the index to remove.
- * @primary_key: if index is a PRIMARY KEY.
- * @table_name: name of the table index to remove from.
- *
- * Proxy the call to the drop_index method on the #GdaServerProvider class
- * to the corresponding provider.
- *
- * Returns: %TRUE if successful, %FALSE otherwise.
- */
-gboolean
-gda_server_provider_drop_index (GdaServerProvider *provider,
-				GdaConnection *cnc,
-				const gchar *index_name,
-				gboolean primary_key,
-				const gchar *table_name)
-{
-	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (index_name != NULL, FALSE);
-	g_return_val_if_fail (table_name != NULL, FALSE);
-	g_return_val_if_fail (CLASS (provider)->drop_index != NULL, FALSE);
-
-	return CLASS (provider)->drop_index (provider, cnc, index_name, primary_key, table_name);
+		cmd = gda_command_new (sql, GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
+		g_free (sql);
+		model = gda_connection_execute_command (cnc, cmd, NULL, error);		
+		gda_command_free (cmd);
+		if (model != GDA_CONNECTION_EXEC_FAILED) {
+			if (model)
+				g_object_unref (model);
+			return TRUE;
+		}
+		else
+			return FALSE;
+	}
 }
 
 /**
@@ -611,6 +636,13 @@ gda_server_provider_execute_command (GdaServerProvider *provider,
 	g_return_val_if_fail (cmd != NULL, NULL);
 	g_return_val_if_fail (CLASS (provider)->execute_command != NULL, NULL);
 
+	{
+		GdaServerProviderInfo *info;
+
+		info = gda_server_provider_get_info (provider, cnc);
+		g_print ("==> %s (Provider %s on cnx %p)\n", cmd->text, info->provider_name, cnc);
+	}
+
 	return CLASS (provider)->execute_command (provider, cnc, cmd, params);
 }
 
@@ -702,7 +734,7 @@ gda_server_provider_rollback_transaction (GdaServerProvider *provider,
 }
 
 /**
- * gda_server_provider_supports
+ * gda_server_provider_supports_feature
  * @provider:
  * @cnc:
  * @feature:
@@ -710,27 +742,30 @@ gda_server_provider_rollback_transaction (GdaServerProvider *provider,
  * Returns:
  */
 gboolean
-gda_server_provider_supports (GdaServerProvider *provider,
+gda_server_provider_supports_feature (GdaServerProvider *provider,
 			      GdaConnection *cnc,
 			      GdaConnectionFeature feature)
 {
 	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
 	if (cnc)
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (CLASS (provider)->supports != NULL, FALSE);
+	g_return_val_if_fail (CLASS (provider)->supports_feature != NULL, FALSE);
 
-	return CLASS (provider)->supports (provider, cnc, feature);
+	return CLASS (provider)->supports_feature (provider, cnc, feature);
 }
 
 /**
  * gda_server_provider_get_schema
- * @provider:
- * @cnc:
- * @schema:
- * @params:
+ * @provider: a #GdaServerProvider object
+ * @cnc: a #GdaConnection object, or %NULL
+ * @schema: the requested kind of information
+ * @params: optional parameters
  *
- * Returns:
- * 
+ * Get a #GdaDataModel containing the requested information. See <link linkend="libgda-provider-get-schema">this section</link> for more 
+ * information on the columns of the returned #GdaDataModel depending on requested @schema, and for the possible
+ * parameters of @params.
+ *
+ * Returns: a new #GdaDataModel, or %NULL if an error occurred.
  */
 GdaDataModel *
 gda_server_provider_get_schema (GdaServerProvider *provider,
@@ -798,20 +833,19 @@ gda_server_provider_fetch_blob_by_id (GdaServerProvider *provider,
 }
 
 /**
- * gda_server_provider_get_data_handler_gda
+ * gda_server_provider_get_data_handler_gtype
  * @provider: a server provider.
  * @cnc: a #GdaConnection object, or %NULL
- * @for_type: a #GdaValueType
+ * @for_type: a #GType
  *
  * Find a #GdaDataHandler object to manipulate data of type @for_type.
  * 
  * Returns: a #GdaDataHandler, or %NULL if the provider does not support the requested @for_type data type 
- *          or @for_type is GDA_VALUE_TYPE_NULL
  */
 GdaDataHandler *
-gda_server_provider_get_data_handler_gda (GdaServerProvider *provider,
+gda_server_provider_get_data_handler_gtype (GdaServerProvider *provider,
 					  GdaConnection *cnc,
-					  GdaValueType for_type)
+					  GType for_type)
 {
 	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), NULL);
 	if (cnc)
@@ -843,7 +877,7 @@ gda_server_provider_get_data_handler_dbms (GdaServerProvider *provider,
 	g_return_val_if_fail (for_type && *for_type, NULL);
 
 	if (CLASS (provider)->get_data_handler)
-		return CLASS (provider)->get_data_handler (provider, cnc, GDA_VALUE_TYPE_UNKNOWN, for_type);
+		return CLASS (provider)->get_data_handler (provider, cnc, G_TYPE_INVALID, for_type);
 	return NULL;
 }
 
@@ -852,28 +886,28 @@ gda_server_provider_get_data_handler_dbms (GdaServerProvider *provider,
  * @provider: a server provider.
  * @cnc: a #GdaConnection object.
  * @string: the SQL string to convert to a value
- * @prefered_type: a #GdaValueType
+ * @prefered_type: a #GType
  *
- * Use @provider to create a new #GdaValue from a single string representation. 
+ * Use @provider to create a new #GValue from a single string representation. 
  *
- * The @prefered_type can optionnaly ask @provider to return a #GdaValue of the requested type 
+ * The @prefered_type can optionnaly ask @provider to return a #GValue of the requested type 
  * (but if such a value can't be created from @string, then %NULL is returned); 
- * pass GDA_VALUE_TYPE_UNKNOWN if any returned type is acceptable.
+ * pass G_TYPE_INVALID if any returned type is acceptable.
  *
- * The returned value is either a new #GdaValue or %NULL in the following cases:
+ * The returned value is either a new #GValue or %NULL in the following cases:
  * - @string cannot be converted to @prefered_type type
  * - the provider does not handle @prefered_type
- * - the provider could not make a #GdaValue from @string
+ * - the provider could not make a #GValue from @string
  *
- * Returns: a new #GdaValue, or %NULL
+ * Returns: a new #GValue, or %NULL
  */
-GdaValue *
+GValue *
 gda_server_provider_string_to_value (GdaServerProvider *provider,
 				     GdaConnection *cnc,
 				     const gchar *string, 
-				     GdaValueType prefered_type, gchar **dbms_type)
+				     GType prefered_type, gchar **dbms_type)
 {
-	GdaValue *retval = NULL;
+	GValue *retval = NULL;
 	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), NULL);
 	if (cnc)
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
@@ -885,8 +919,8 @@ gda_server_provider_string_to_value (GdaServerProvider *provider,
 		GdaDataHandler *dh;
 		gint i;
 
-		if (prefered_type) {
-			dh = gda_server_provider_get_data_handler_gda (provider, cnc, prefered_type);
+		if (prefered_type != G_TYPE_INVALID) {
+			dh = gda_server_provider_get_data_handler_gtype (provider, cnc, prefered_type);
 			if (dh) {
 				retval = gda_data_handler_get_value_from_sql (dh, string, prefered_type);
 				if (retval) {
@@ -897,36 +931,42 @@ gda_server_provider_string_to_value (GdaServerProvider *provider,
 						gda_value_free (retval);
 						retval = NULL;
 					}
+					else {
+						if (dbms_type)
+							*dbms_type = (gchar *) gda_server_provider_get_default_dbms_type (provider, 
+														cnc, prefered_type);
+					}
+
 					g_free (tmp);
 				}
 			}
 		}
 		else {
 			/* test all the possible data types and see if we have a match */
-			GdaValueType types[] = {GDA_VALUE_TYPE_TINYUINT,
-						GDA_VALUE_TYPE_SMALLUINT,
-						GDA_VALUE_TYPE_UINTEGER,
-						GDA_VALUE_TYPE_BIGUINT,
-						
-						GDA_VALUE_TYPE_TINYINT,
-						GDA_VALUE_TYPE_SMALLINT,
-						GDA_VALUE_TYPE_INTEGER,
-						GDA_VALUE_TYPE_BIGINT,
-						
-						GDA_VALUE_TYPE_SINGLE,
-						GDA_VALUE_TYPE_DOUBLE,
-						GDA_VALUE_TYPE_NUMERIC,
-						
-						GDA_VALUE_TYPE_BOOLEAN,
-						GDA_VALUE_TYPE_TIME,
-						GDA_VALUE_TYPE_DATE,
-						GDA_VALUE_TYPE_TIMESTAMP,
-						GDA_VALUE_TYPE_GEOMETRIC_POINT,
-						GDA_VALUE_TYPE_STRING,
-						GDA_VALUE_TYPE_BINARY};
+			GType types[] = {G_TYPE_UCHAR,
+					 GDA_TYPE_USHORT,
+					 G_TYPE_UINT,
+					 G_TYPE_UINT64,
+					 
+					 G_TYPE_CHAR,
+					 GDA_TYPE_SHORT,
+					 G_TYPE_INT,
+					 G_TYPE_INT64,
+					 
+					 G_TYPE_FLOAT,
+					 G_TYPE_DOUBLE,
+					 GDA_TYPE_NUMERIC,
+					 
+					 G_TYPE_BOOLEAN,
+					 GDA_TYPE_TIME,
+					 G_TYPE_DATE,
+					 GDA_TYPE_TIMESTAMP,
+					 GDA_TYPE_GEOMETRIC_POINT,
+					 G_TYPE_STRING,
+					 GDA_TYPE_BINARY};
 
-			for (i = 0; !retval && (i <= (sizeof(types)/sizeof (GdaValueType)) - 1); i++) {
-				dh = gda_server_provider_get_data_handler_gda (provider, cnc, types [i]);
+			for (i = 0; !retval && (i <= (sizeof(types)/sizeof (GType)) - 1); i++) {
+				dh = gda_server_provider_get_data_handler_gtype (provider, cnc, types [i]);
 				if (dh) {
 					retval = gda_data_handler_get_value_from_sql (dh, string, types [i]);
 					if (retval) {
@@ -937,6 +977,11 @@ gda_server_provider_string_to_value (GdaServerProvider *provider,
 							gda_value_free (retval);
 							retval = NULL;
 						}
+						else {
+							if (dbms_type)
+								*dbms_type = (gchar *) gda_server_provider_get_default_dbms_type (provider, 
+															cnc, types[i]);
+						}
 						g_free (tmp);
 					}
 				}
@@ -952,16 +997,16 @@ gda_server_provider_string_to_value (GdaServerProvider *provider,
  * gda_server_provider_value_to_sql_string
  * @provider: a server provider.
  * @cnc: a #GdaConnection object, or %NULL
- * @from: #GdaValue to convert from
+ * @from: #GValue to convert from
  *
- * Produces a fully quoted and escaped string from a GdaValue
+ * Produces a fully quoted and escaped string from a GValue
  *
  * Returns: escaped and quoted value or NULL if not supported.
  */
 gchar *
 gda_server_provider_value_to_sql_string (GdaServerProvider *provider,
 					 GdaConnection *cnc,
-					 GdaValue *from)
+					 GValue *from)
 {
 	GdaDataHandler *dh;
 
@@ -970,7 +1015,7 @@ gda_server_provider_value_to_sql_string (GdaServerProvider *provider,
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 	g_return_val_if_fail (from != NULL, NULL);
 
-	dh = gda_server_provider_get_data_handler_gda (provider, cnc, GDA_VALUE_TYPE (from));
+	dh = gda_server_provider_get_data_handler_gtype (provider, cnc, G_VALUE_TYPE (from));
 	if (dh)
 		return gda_data_handler_get_sql_from_value (dh, from);
 	else
@@ -981,23 +1026,27 @@ gda_server_provider_value_to_sql_string (GdaServerProvider *provider,
  * gda_server_provider_get_default_dbms_type
  * @provider: a server provider.
  * @cnc: a #GdaConnection object or %NULL
- * @gda_type: a #GdaValueType value type
+ * @type: a #GType value type
  *
- * Get the name of the most common data type which has @gda_type GDA type
+ * Get the name of the most common data type which has @type type.
+
+ * The returned value may be %NULL either if the provider does not implement that method, or if
+ * there is no DBMS data type which could contain data of the @gda_type type (for example %NULL may be
+ * returned if a DBMS has integers only up to 4 bytes and a G_TYPE_INT64 is requested).
  *
  * Returns: the name of the DBMS type, or %NULL
  */
 const gchar *
 gda_server_provider_get_default_dbms_type (GdaServerProvider *provider,
 					   GdaConnection *cnc,
-					   GdaValueType gda_type)
+					   GType type)
 {
 	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), NULL);
 	if (cnc)
 		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 	
 	if (CLASS (provider)->get_def_dbms_type)
-		return (CLASS (provider)->get_def_dbms_type)(provider, cnc, gda_type);
+		return (CLASS (provider)->get_def_dbms_type)(provider, cnc, type);
 	else
 		return NULL;
 }
