@@ -1,5 +1,5 @@
 /* GDA IBMDB2 Provider
- * Copyright (C) 2002 - 2005 The GNOME Foundation
+ * Copyright (C) 2002 - 2006 The GNOME Foundation
  *
  * AUTHORS:
  *         Holger Thon <holger.thon@gnome-db-org>
@@ -29,6 +29,8 @@
 #include "gda-ibmdb2.h"
 #include "gda-ibmdb2-recordset.h"
 
+#include <libgda/sql-delimiter/gda-sql-delimiter.h>
+
 #define PARENT_TYPE GDA_TYPE_SERVER_PROVIDER
 
 #define OBJECT_DATA_IBMDB2_HANDLE "GDA_IBMDB2_IBMDB2Handle"
@@ -56,12 +58,6 @@ static const gchar *gda_ibmdb2_provider_get_database (GdaServerProvider *provide
 static gboolean gda_ibmdb2_provider_change_database (GdaServerProvider *provider,
                                                      GdaConnection *cnc,
                                                      const gchar *name);
-static gboolean gda_ibmdb2_provider_create_database_cnc (GdaServerProvider *provider,
-                                                         GdaConnection *cnc,
-                                                         const gchar *name);
-static gboolean gda_ibmdb2_provider_drop_database_cnc (GdaServerProvider *provider,
-						       GdaConnection *cnc,
-						       const gchar *name);
 static GList *gda_ibmdb2_provider_execute_command (GdaServerProvider *provider,
                                                    GdaConnection *cnc,
                                                    GdaCommand *cmd,
@@ -364,55 +360,6 @@ gda_ibmdb2_provider_single_command (GdaIBMDB2ConnectionData *conn_data,
 	return TRUE;
 }
 
-
-static gboolean
-gda_ibmdb2_provider_create_database_cnc (GdaServerProvider *provider,                                                    
-				         GdaConnection *cnc,
-                                         const gchar *name)
-{
-	GdaIBMDB2Provider *db2_prov = (GdaIBMDB2Provider *) provider;
-	GdaIBMDB2ConnectionData *conn_data = NULL;
-	gchar *sql;
-	gboolean retval;
-
-	g_return_val_if_fail (GDA_IS_IBMDB2_PROVIDER (db2_prov), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	
-        conn_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_IBMDB2_HANDLE);
-        g_return_val_if_fail (conn_data != NULL, FALSE);
-	
-	sql = g_strdup_printf("CREATE DATABASE %s", name);
-	retval = gda_ibmdb2_provider_single_command (conn_data, cnc, sql);	
-	g_free(sql);
-	
-	return retval;
-}
-
-
-
-static gboolean
-gda_ibmdb2_provider_drop_database_cnc (GdaServerProvider *provider,
-				       GdaConnection *cnc,
-				       const gchar *name)
-{
-	GdaIBMDB2Provider *db2_prov = (GdaIBMDB2Provider *) provider;
-	GdaIBMDB2ConnectionData *conn_data = NULL;
-	gchar *sql;
-	gboolean retval;
-
-	g_return_val_if_fail (GDA_IS_IBMDB2_PROVIDER (db2_prov), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	
-        conn_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_IBMDB2_HANDLE);
-        g_return_val_if_fail (conn_data != NULL, FALSE);
-	
-	sql = g_strdup_printf("DROP DATABASE %s", name);
-	retval = gda_ibmdb2_provider_single_command (conn_data, cnc, sql);	
-	g_free(sql);
-	
-	return retval;
-}
-
 static const gboolean 
 gda_ibmdb2_provider_update_database (GdaIBMDB2ConnectionData *conn_data)
 {
@@ -453,7 +400,7 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql, GdaC
 		return NULL;
 	}
 	
-	arr = g_strsplit (sql, ";", 0);
+	arr = gda_delimiter_split_sql (sql);
 	
         if (arr) {
  		gint n = 0;     
@@ -780,10 +727,11 @@ static GdaDataModel
 
 	for (i = 0; conn_data->rc != SQL_NO_DATA_FOUND; i++)
 	{
-	    
+		GValue *tmpval;
 		str = g_strndup(alias, alias_len);
 	    
-	    	value_list = g_list_append (NULL, gda_value_new_string (str));
+		g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), str);
+	    	value_list = g_list_append (NULL, tmpval);
 	    	gda_data_model_append_values (GDA_DATA_MODEL (recset), value_list);
 	   
 	    	g_free(str);
@@ -1061,7 +1009,7 @@ gda_ibmdb2_provider_class_init (GdaIBMDB2ProviderClass *klass)
 	provider_class->get_version = gda_ibmdb2_provider_get_version;
 	provider_class->get_server_version = gda_ibmdb2_provider_get_server_version;
 	provider_class->get_info = NULL;
-	provider_class->supports = gda_ibmdb2_provider_supports;
+	provider_class->supports_feature = gda_ibmdb2_provider_supports;
 	provider_class->get_schema = gda_ibmdb2_provider_get_schema;
 
 	provider_class->get_data_handler = NULL;
@@ -1069,20 +1017,14 @@ gda_ibmdb2_provider_class_init (GdaIBMDB2ProviderClass *klass)
 	provider_class->get_def_dbms_type = NULL;
 
 	provider_class->open_connection = gda_ibmdb2_provider_open_connection;
-	provider_class->reset_connection = NULL;
 	provider_class->close_connection = gda_ibmdb2_provider_close_connection;
 	provider_class->get_database = gda_ibmdb2_provider_get_database;
 	provider_class->change_database = gda_ibmdb2_provider_change_database;
 
-	provider_class->get_specs = NULL;
-	provider_class->perform_action_params = NULL;
-
-	provider_class->create_database_cnc = gda_ibmdb2_provider_create_database_cnc;
-	provider_class->drop_database_cnc = gda_ibmdb2_provider_drop_database_cnc;
-	provider_class->create_table = NULL;
-	provider_class->drop_table = NULL;
-	provider_class->create_index = NULL;
-	provider_class->drop_index = NULL;
+	provider_class->supports_operation = NULL;
+        provider_class->create_operation = NULL;
+        provider_class->render_operation = NULL;
+        provider_class->perform_operation = NULL;
 
 	provider_class->execute_command = gda_ibmdb2_provider_execute_command;
 	provider_class->get_last_insert_id = NULL;
