@@ -1,5 +1,5 @@
 /* GDA Sybase provider
- * Copyright (C) 1998 - 2005 The GNOME Foundation.
+ * Copyright (C) 1998 - 2006 The GNOME Foundation.
  *
  * AUTHORS:
  *         Mike Wingert <wingert.3@postbox.acs.ohio-state.edu>
@@ -35,6 +35,8 @@
 #include "gda-sybase-types.h"
 #include <libgda/gda-data-model-private.h>
 
+#include <libgda/sql-delimiter/gda-sql-delimiter.h>
+
 #define PARENT_TYPE GDA_TYPE_SERVER_PROVIDER
 
 static void gda_sybase_provider_class_init (GdaSybaseProviderClass *klass);
@@ -60,12 +62,6 @@ static gboolean gda_sybase_provider_change_database (GdaServerProvider *provider
 static gboolean gda_sybase_provider_commit_transaction (GdaServerProvider *provider,
                                                         GdaConnection *cnc,
                                                         GdaTransaction *xaction);
-static gboolean gda_sybase_provider_create_database_cnc (GdaServerProvider *provider,
-                                                         GdaConnection *cnc,
-                                                         const gchar *name);
-static gboolean gda_sybase_provider_drop_database_cnc (GdaServerProvider *provider,
-						       GdaConnection *cnc,
-						       const gchar *name);
 static GList *gda_sybase_provider_execute_command (GdaServerProvider *provider,
                                                    GdaConnection *cnc,
                                                    GdaCommand *cmd,
@@ -111,7 +107,7 @@ gda_sybase_provider_class_init (GdaSybaseProviderClass *klass)
 	provider_class->get_version = gda_sybase_provider_get_version;
 	provider_class->get_server_version = gda_sybase_provider_get_server_version;
 	provider_class->get_info = NULL;
-	provider_class->supports = gda_sybase_provider_supports;
+	provider_class->supports_feature = gda_sybase_provider_supports;
 	provider_class->get_schema = gda_sybase_provider_get_schema;
 
 	provider_class->get_data_handler = NULL;
@@ -119,20 +115,14 @@ gda_sybase_provider_class_init (GdaSybaseProviderClass *klass)
 	provider_class->get_def_dbms_type = NULL;
 
 	provider_class->open_connection = gda_sybase_provider_open_connection;
-	provider_class->reset_connection = NULL;
 	provider_class->close_connection = gda_sybase_provider_close_connection;
 	provider_class->get_database = gda_sybase_provider_get_database;
 	provider_class->change_database = gda_sybase_provider_change_database;
 
-	provider_class->get_specs = NULL;
-	provider_class->perform_action_params = NULL;
-
-	provider_class->create_database_cnc = gda_sybase_provider_create_database_cnc;
-	provider_class->drop_database_cnc = gda_sybase_provider_drop_database_cnc;
-	provider_class->create_table = NULL;
-	provider_class->drop_table = NULL;
-	provider_class->create_index = NULL;
-	provider_class->drop_index = NULL;
+	provider_class->supports_operation = NULL;
+        provider_class->create_operation = NULL;
+        provider_class->render_operation = NULL;
+        provider_class->perform_operation = NULL;
 
 	provider_class->execute_command = gda_sybase_provider_execute_command;
 	provider_class->get_last_insert_id = NULL;
@@ -585,22 +575,6 @@ gda_sybase_provider_commit_transaction (GdaServerProvider *provider,
 	return ( gda_sybase_execute_cmd(cnc,"commit transaction") );		
 }
 
-static gboolean 
-gda_sybase_provider_create_database_cnc (GdaServerProvider *provider,
-                                         GdaConnection *cnc,
-                                         const gchar *name)
-{
-	return FALSE;
-}
-
-static gboolean 
-gda_sybase_provider_drop_database_cnc (GdaServerProvider *provider,
-				       GdaConnection *cnc,
-				       const gchar *name)
-{
-	return FALSE;
-}
-
 static GList *
 gda_sybase_provider_execute_command (GdaServerProvider *provider,
                                      GdaConnection *cnc,
@@ -667,7 +641,7 @@ gda_sybase_provider_process_sql_commands(GList         *reclist,
 
 	// FIXME: ; in values bug
 	//        e.g. SELECT * from x where y LIKE '%foo; bar%'
-	arr = g_strsplit (sql, ";", 0);
+	arr = gda_delimiter_split_sql (sql);
 	if (arr) {
 		gint n = 0;
 		while (arr[n]) {
@@ -1131,16 +1105,17 @@ gda_sybase_provider_get_types (GdaConnection *cnc,
 	for (i = 0; i < GDA_SYBASE_TYPE_CNT; i++) {
 		if (gda_sybase_type_list[i].name != NULL) {
 			GList *value_list = NULL;
+			GValue *tmpval;
 			
-			value_list = g_list_append (value_list,
-						    gda_value_new_string (gda_sybase_type_list[i].name));
+			g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), gda_sybase_type_list[i].name);
+			value_list = g_list_append (value_list, tmpval);
+
 			// FIXME: owner
-			value_list = g_list_append (value_list,
-						    gda_value_new_string (""));
-			value_list = g_list_append (value_list, 
-						    gda_value_new_string (""));
-			value_list = g_list_append (value_list,
-						    gda_value_new_gdatype (gda_sybase_type_list[i].gda_type));
+			value_list = g_list_append (value_list, gda_value_new_null ());
+			value_list = g_list_append (value_list, gda_value_new_null ());
+
+			g_value_set_ulong (tmpval = gda_value_new (G_TYPE_ULONG), gda_sybase_type_list[i].gda_type);
+			value_list = g_list_append (value_list, tmpval);
 			
 			gda_data_model_append_values (GDA_DATA_MODEL (recset), value_list);
 			g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
@@ -1271,11 +1246,11 @@ gda_sybase_provider_get_server_version (GdaServerProvider *provider,
 		if (model) {
 			if ((gda_data_model_get_n_columns (model) == 1)
 			    && (gda_data_model_get_n_rows (model) == 1)) {
-				GdaValue *value;
+				GValue *value;
 				
-				value = (GdaValue *) gda_data_model_get_value_at (model, 
+				value = (GValue *) gda_data_model_get_value_at (model, 
 										  0, 0);
-				scnc->server_version = gda_value_stringify ((GdaValue *) value);
+				scnc->server_version = gda_value_stringify ((GValue *) value);
 			}
 			g_object_unref (model);
 			model = NULL;
